# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Glissando_01():

    target = Container([Note(0, 1), Note(0, 1)])
    glissando = spannertools.Glissando()
    attach(glissando, target[:])
    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Glissando_02():

    string = r'{ c \glissando }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Glissando_03():

    string = r'{ \glissando c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach


def make_score(tuplet_duration, row_count, column_count):
    r'''Makes score.
    '''

    score = scoretools.Score()
    rows_of_nested_tuplets = \
        abjad.demos.ferneyhough.make_rows_of_nested_tuplets(
        tuplet_duration, row_count, column_count)
    for row_of_nested_tuplets in rows_of_nested_tuplets:
        staff = scoretools.Staff(row_of_nested_tuplets)
        staff.context_name = 'RhythmicStaff'
        time_signature = indicatortools.TimeSignature((1, 4))
        attach(time_signature, staff)
        score.append(staff)
    return score


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental_abbreviation_01():
    accidental = pitchtools.Accidental('s')
    assert accidental.abbreviation == 's'


def test_pitchtools_Accidental_abbreviation_02():
    accidental = pitchtools.Accidental('')
    assert accidental.abbreviation == ''


def test_pitchtools_Accidental_abbreviation_03():
    accidental = pitchtools.Accidental()
    assert accidental.abbreviation == ''


def test_pitchtools_Accidental_abbreviation_04():

    assert pitchtools.Accidental('').abbreviation == ''
    assert pitchtools.Accidental('bb').abbreviation == 'ff'
    assert pitchtools.Accidental('b~').abbreviation == 'tqf'
    assert pitchtools.Accidental('b').abbreviation == 'f'
    assert pitchtools.Accidental('~').abbreviation == 'qf'
    assert pitchtools.Accidental('##').abbreviation == 'ss'
    assert pitchtools.Accidental('#+').abbreviation == 'tqs'
    assert pitchtools.Accidental('#').abbreviation == 's'
    assert pitchtools.Accidental('+').abbreviation == 'qs'



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools.Instrument import Instrument


class AltoFlute(Instrument):
    r'''An alto flute.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> alto_flute = instrumenttools.AltoFlute()
        >>> attach(alto_flute, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { "Alto flute" }
            \set Staff.shortInstrumentName = \markup { "Alt. fl." }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='alto flute',
        short_instrument_name='alt. fl.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range='[G3, G6]',
        sounding_pitch_of_written_middle_c='G3',
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'wind player',
            'flautist',
            'flutist',
            ])
        self._starting_clefs = type(self.allowable_clefs)(['treble'])

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats alto flute.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> alto_flute = instrumenttools.AltoFlute()
                >>> print(format(alto_flute))
                instrumenttools.AltoFlute(
                    instrument_name='alto flute',
                    short_instrument_name='alt. fl.',
                    instrument_name_markup=markuptools.Markup(
                        contents=['Alto flute'],
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=['Alt. fl.'],
                        ),
                    allowable_clefs=instrumenttools.ClefList(
                        [
                            indicatortools.Clef(
                                name='treble',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[G3, G6]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('g'),
                    )

        Returns string.
        '''
        superclass = super(AltoFlute, self)
        return superclass.__format__(format_specification=format_specification)

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets alto flute's allowable clefs.

        ..  container:: example

            ::

                >>> alto_flute.allowable_clefs
                ClefList([Clef(name='treble')])

            ::

                >>> show(alto_flute.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets alto flute's name.

        ..  container:: example

            ::

                >>> alto_flute.instrument_name
                'alto flute'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets alto flute's instrument name markup.

        ..  container:: example

            ::

                >>> alto_flute.instrument_name_markup
                Markup(contents=['Alto flute'])

            ::

                >>> show(alto_flute.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets alto flute's range.

        ..  container:: example

            ::

                >>> alto_flute.pitch_range
                PitchRange(range_string='[G3, G6]')

            ::

                >>> show(alto_flute.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets alto flute's short instrument name.

        ..  container:: example

            ::

                >>> alto_flute.short_instrument_name
                'alt. fl.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets alto flute's short instrument name markup.

        ..  container:: example

            ::

                >>> alto_flute.short_instrument_name_markup
                Markup(contents=['Alt. fl.'])

            ::

                >>> show(alto_flute.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of alto flute's written middle C.

        ..  container:: example

            ::

                >>> alto_flute.sounding_pitch_of_written_middle_c
                NamedPitch('g')

            ::

                >>> show(alto_flute.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_DuratedComplexBeam_span_beam_count_01():
    r'''1-beam span between adjacent groups of 1/16th notes.
    '''

    container = Container("c'16 d'16 e'16 f'16")
    beam = spannertools.DuratedComplexBeam(
        durations=[(1, 8), (1, 8)],
        span_beam_count=1,
        )
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #1
            d'16
            \set stemLeftBeamCount = #1
            \set stemRightBeamCount = #2
            e'16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 ]
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_spannertools_DuratedComplexBeam_span_beam_count_02():
    r'''2-beam span between adjacent groups of 1/16th notes.
    '''

    container = Container("c'16 d'16 e'16 f'16")
    beam = spannertools.DuratedComplexBeam(
        durations=[(1, 8), (1, 8)],
        span_beam_count=2,
        )
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            d'16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 ]
        }
        '''
        )

    assert inspect_(container).is_well_formed()


# -*- coding: utf-8 -*-


def list_all_abjad_functions(modules=None):
    r'''Lists all public functions defined in Abjad.

    ::

        >>> all_functions = documentationtools.list_all_abjad_functions()

    '''
    from abjad.tools import documentationtools
    return documentationtools.list_all_functions(modules='abjad')



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__functions__grace_01():

    target = Container([
        Note("c'4"),
        Note("d'4"),
        Note("e'2")
    ])

    grace = scoretools.GraceContainer([
        Note("g''16"),
        Note("fs''16")
    ])

    attach(grace, target[2])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4
            d'4
            \grace {
                g''16
                fs''16
            }
            e'2
        }
        '''
        )

    string = r"{ c'4 d'4 \grace { g''16 fs''16} e'2 }"
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools.quantizationtools.QEvent import QEvent


class PitchedQEvent(QEvent):
    r'''A ``QEvent`` which indicates the onset of a period of pitched material
    in a ``QEventSequence``.

    ::

        >>> pitches = [0, 1, 4]
        >>> q_event = quantizationtools.PitchedQEvent(1000, pitches)
        >>> print(format(q_event, 'storage'))
        quantizationtools.PitchedQEvent(
            offset=durationtools.Offset(1000, 1),
            pitches=(
                pitchtools.NamedPitch("c'"),
                pitchtools.NamedPitch("cs'"),
                pitchtools.NamedPitch("e'"),
                ),
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_attachments',
        '_index',
        '_offset',
        '_pitches',
        )

    ### INITIALIZER ###

    def __init__(self, offset=0, pitches=None, attachments=None, index=None):
        QEvent.__init__(self, offset=offset, index=index)
        pitches = pitches or []
        pitches = tuple([pitchtools.NamedPitch(x) for x in pitches])
        if attachments is None:
            attachments = ()
        else:
            attachments = tuple(attachments)
        self._pitches = pitches
        self._attachments = attachments

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a pitched q-event with offset, pitches,
        attachments and index equal to those of this pitched q-event. Otherwise
        false.

        Returns true or false.
        '''
        if type(self) == type(argument) and \
            self.offset == argument.offset and \
            self.pitches == argument.pitches and \
            self.attachments == argument.attachments and \
            self.index == argument.index:
            return True
        return False

    def __hash__(self):
        r'''Hashes pitched q-event.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(PitchedQEvent, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def attachments(self):
        r'''Attachments of pitched q-event.
        '''
        return self._attachments

    @property
    def pitches(self):
        r'''Pitches of pitched q-event.
        '''
        return self._pitches



# -*- coding: utf-8 -*-


def is_fraction_equivalent_pair(argument):
    r'''Is true when `argument` is an integer-equivalent pair of numbers
    excluding 0 as the second term. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_fraction_equivalent_pair((2, 3))
            True

        ::

            >>> mathtools.is_fraction_equivalent_pair((2, 0))
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return (
        mathtools.is_integer_equivalent_pair(argument) and
        not argument[1] == 0
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.quantizationtools import *


def test_quantizationtools_QGrid_subdivide_leaves_01():

    q_grid = QGrid()

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)

    q_grid.leaves[0].q_event_proxies.extend([a, b, c, d])
    q_grid.leaves[1].q_event_proxies.extend([e, f, g])

    assert q_grid.root_node.rtm_format == '1'

    q_events = q_grid.subdivide_leaves([(0, (1, 1))])

    assert q_events == [a, b, c, d, e, f]
    assert q_grid.root_node.rtm_format == '(1 (1 1))'

    q_grid.leaves[0].q_event_proxies.extend([a, b])
    q_grid.leaves[1].q_event_proxies.extend([c, d, e])
    q_grid.leaves[2].q_event_proxies.append(f)

    q_events = q_grid.subdivide_leaves([(0, (1, 1)), (1, (1, 1))])

    assert q_events == [a, b, c, d, e, f]
    assert q_grid.root_node.rtm_format == '(1 ((1 (1 1)) (1 (1 1))))'


def test_quantizationtools_QGrid_subdivide_leaves_02():

    q_grid = QGrid()

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)

    q_grid.leaves[0].q_event_proxies.extend([a, b, c, d])
    q_grid.leaves[1].q_event_proxies.extend([e, f, g])

    assert q_grid.root_node.rtm_format == '1'

    q_events = q_grid.subdivide_leaves([(0, (1, 1))])

    assert q_events == [a, b, c, d, e, f]
    assert q_grid.root_node.rtm_format == '(1 (1 1))'

    q_grid.leaves[0].q_event_proxies.extend([a, b])
    q_grid.leaves[1].q_event_proxies.extend([c, d, e])
    q_grid.leaves[2].q_event_proxies.append(f)

    q_events = q_grid.subdivide_leaves([(0, (3, 4, 5))])

    assert q_events == [a, b, c]
    assert q_grid.root_node.rtm_format == '(1 ((1 (3 4 5)) 1))'



# -*- coding: utf-8 -*-
'''The Abjad expressiontools package imports no other Abjad packages.
'''
from abjad.tools import systemtools


systemtools.ImportManager.import_structured_package(
    __path__[0],
    globals(),
    )

_documentation_section = 'core'



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_SilentQEvent___eq___01():

    a = quantizationtools.SilentQEvent(1000)
    b = quantizationtools.SilentQEvent(1000)

    assert a == b


def test_quantizationtools_SilentQEvent___eq___02():

    a = quantizationtools.SilentQEvent(1000)
    b = quantizationtools.SilentQEvent(1000, ['foo', 'bar', 'baz'])
    c = quantizationtools.SilentQEvent(9999)

    assert a != b
    assert a != c



# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools


def test_rhythmtreetools_RhythmTreeContainer___eq___01():

    a = rhythmtreetools.RhythmTreeContainer(children=[])
    b = rhythmtreetools.RhythmTreeContainer(children=[])

    assert format(a) == format(b)
    assert a != b


def test_rhythmtreetools_RhythmTreeContainer___eq___02():

    a = rhythmtreetools.RhythmTreeContainer(children=[
        rhythmtreetools.RhythmTreeLeaf()
        ])
    b = rhythmtreetools.RhythmTreeContainer(children=[
        rhythmtreetools.RhythmTreeLeaf()
        ])

    assert format(a) == format(b)
    assert a != b


def test_rhythmtreetools_RhythmTreeContainer___eq___03():

    a = rhythmtreetools.RhythmTreeContainer(children=[])
    b = rhythmtreetools.RhythmTreeContainer(preprolated_duration=2, children=[])
    c = rhythmtreetools.RhythmTreeContainer(preprolated_duration=1, children=[
        rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
        ])
    d = rhythmtreetools.RhythmTreeContainer(preprolated_duration=2, children=[
        rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
        ])
    e = rhythmtreetools.RhythmTreeContainer(preprolated_duration=2, children=[
        rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
        ])

    assert a != b
    assert a != c
    assert a != d
    assert a != e
    assert b != c
    assert b != d
    assert b != e
    assert c != d
    assert c != e
    assert d != e



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch___float___01():
    r'''Returns pitch number of 12-ET named pitch as float.
    '''

    named_pitch = NamedPitch(13)
    assert isinstance(float(named_pitch), float)
    assert float(named_pitch) == 13.0


def test_pitchtools_NamedPitch___float___02():
    r'''Returns pitch number of 24-ET named pitch as float.
    '''

    named_pitch = NamedPitch(13.5)
    assert isinstance(float(named_pitch), float)
    assert float(named_pitch) == 13.5



# -*- coding: utf-8 -*-
from abjad.tools import patterntools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class SilenceMask(AbjadValueObject):
    r'''Silence mask.

    ..  container:: example

        ::

            >>> pattern = patterntools.select_every([0, 1, 7], period=16)
            >>> mask = rhythmmakertools.SilenceMask(pattern)

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[0, 1, 7],
                    period=16,
                    ),
                )

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Masks'

    __slots__ = (
        '_pattern',
        '_use_multimeasure_rests',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        pattern=None,
        use_multimeasure_rests=None,
        ):
        from abjad.tools import rhythmmakertools
        prototype = (
            patterntools.Pattern,
            patterntools.CompoundPattern,
            )
        if pattern is None:
            pattern = patterntools.select_all()
        assert isinstance(pattern, prototype), repr(pattern)
        self._pattern = pattern
        if use_multimeasure_rests is not None:
            assert isinstance(use_multimeasure_rests, type(True))
        self._use_multimeasure_rests = use_multimeasure_rests

    ### PUBLIC PROPERTIES ###

    @property
    def pattern(self):
        r'''Gets pattern.

        Returns pattern.
        '''
        return self._pattern

    @property
    def use_multimeasure_rests(self):
        r'''Is true when silence mask should use multimeasure rests.

        ..  container:: example

            Without multimeasure rests:

            ::

            
                >>> mask = rhythmmakertools.SilenceMask(
                ...     patterntools.select_every([0, 1, 7], period=16),
                ...     use_multimeasure_rests=False,
                ...     )

            ::

                >>> mask.use_multimeasure_rests
                False

        ..  container:: example

            With multimeasure rests:

            ::

                >>> mask = rhythmmakertools.SilenceMask(
                ...     patterntools.select_every([0, 1, 7], period=16),
                ...     use_multimeasure_rests=True,
                ...     )

            ::

                >>> mask.use_multimeasure_rests
                True

        Set to true, false or none.
        '''
        return self._use_multimeasure_rests



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch___cmp___01():

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = pitchtools.NumberedPitch(12)

    assert not pitch_1 <  pitch_2
    assert      pitch_1 <= pitch_2
    assert      pitch_1 == pitch_2
    assert not pitch_1 != pitch_2
    assert not pitch_1 >  pitch_2
    assert      pitch_1 >= pitch_2


def test_pitchtools_NumberedPitch___cmp___02():

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = pitchtools.NumberedPitch(13)

    assert not pitch_2 <  pitch_1
    assert not pitch_2 <= pitch_1
    assert not pitch_2 == pitch_1
    assert      pitch_2 != pitch_1
    assert      pitch_2 >  pitch_1
    assert      pitch_2 >= pitch_1


def test_pitchtools_NumberedPitch___cmp___03():

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = 12

    assert not pitch_1 <  pitch_2
    assert      pitch_1 <= pitch_2
    assert      pitch_1 == pitch_2
    assert not pitch_1 != pitch_2
    assert not pitch_1 >  pitch_2
    assert      pitch_1 >= pitch_2


def test_pitchtools_NumberedPitch___cmp___04():

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = 13

    assert not pitch_2 <  pitch_1
    assert not pitch_2 <= pitch_1
    assert not pitch_2 == pitch_1
    assert      pitch_2 != pitch_1
    assert      pitch_2 >  pitch_1
    assert      pitch_2 >= pitch_1



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchRange___setattr___01():
    r'''Pitch ranges are immutable.
    '''

    pitch_range = pitchtools.PitchRange.from_pitches(-12, 36)

    assert pytest.raises(AttributeError, "pitch_range.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


class Aggregate(abctools.AbjadValueObject):

    __slots__ = ('_pitch_segment', '_ratio')

    def __init__(self, pitch_segment=None, ratio=None):
        if pitch_segment is not None:
            pitch_segment = pitchtools.PitchSegment(pitch_segment)
        self._pitch_segment = pitch_segment
        if ratio is not None:
            ratio = mathtools.Ratio(ratio)
        self._ratio = ratio

    @property
    def pitch_segment(self):
        return self._pitch_segment

    @property
    def ratio(self):
        return self._ratio


def test_topleveltools_new_01():

    old_aggregate = Aggregate(
        pitch_segment=pitchtools.PitchSegment('c d e f'),
        ratio=mathtools.Ratio([1, 2, 3])
        )

    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((1, 2, 3)),
            )
        ''')

    new_aggregate = new(old_aggregate)

    assert new_aggregate is not old_aggregate
    assert new_aggregate == old_aggregate
    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((1, 2, 3)),
            )
        ''')
    assert format(new_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((1, 2, 3)),
            )
        ''')


def test_topleveltools_new_02():

    old_aggregate = Aggregate(
        pitch_segment=pitchtools.PitchSegment('c d e f'),
        ratio=mathtools.Ratio([1, 2, 3])
        )

    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((1, 2, 3)),
            )
        ''')

    new_aggregate = new(
        old_aggregate,
        ratio=(4, 5),
        )

    assert new_aggregate is not old_aggregate
    assert new_aggregate != old_aggregate
    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((1, 2, 3)),
            )
        ''')
    assert format(new_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((4, 5)),
            )
        ''')


def test_topleveltools_new_03():

    old_aggregate = Aggregate(
        pitch_segment=pitchtools.PitchSegment('c d e f'),
        )

    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')

    new_aggregate = new(
        old_aggregate,
        pitch_segment='af bf df',
        ratio=(5, 4),
        )

    assert new_aggregate is not old_aggregate
    assert new_aggregate != old_aggregate
    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')
    assert format(new_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('af'),
                    pitchtools.NamedPitch('bf'),
                    pitchtools.NamedPitch('df'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((5, 4)),
            )
        ''')


def test_topleveltools_new_04():

    old_aggregate = Aggregate(
        pitch_segment=pitchtools.PitchSegment('c d e f'),
        )

    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')

    new_aggregate = new(
        old_aggregate,
        pitch_segment__rotate=2,
        ratio=[4, 5],
        )

    assert new_aggregate is not old_aggregate
    assert new_aggregate != old_aggregate
    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')
    assert format(new_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((4, 5)),
            )
        ''')


def test_topleveltools_new_05():

    old_aggregate = Aggregate(
        pitch_segment=pitchtools.PitchSegment('c d e f'),
        )

    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')

    new_aggregate = new(
        old_aggregate,
        ratio=[4, 5],
        pitch_segment__rotate=2,
        )

    assert new_aggregate is not old_aggregate
    assert new_aggregate != old_aggregate
    assert format(old_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            )
        ''')
    assert format(new_aggregate) == stringtools.normalize(
        r'''
        test_topleveltools_new.Aggregate(
            pitch_segment=pitchtools.PitchSegment(
                (
                    pitchtools.NamedPitch('e'),
                    pitchtools.NamedPitch('f'),
                    pitchtools.NamedPitch('c'),
                    pitchtools.NamedPitch('d'),
                    ),
                item_class=pitchtools.NamedPitch,
                ),
            ratio=mathtools.Ratio((4, 5)),
            )
        ''')



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_TimeSignature___repr___01():
    r'''Time signature returns nonempty string repr.
    '''

    time_signature_repr = TimeSignature((3, 8)).__repr__()
    assert isinstance(time_signature_repr, str) and \
        0 < len(time_signature_repr)


def test_indicatortools_TimeSignature___repr___02():
    r'''Repr is evaluable.
    '''

    time_signature_1 = TimeSignature((3, 8), partial=Duration(1, 8))
    time_signature_2 = eval(repr(time_signature_1))

    assert time_signature_1 == time_signature_2



# -*- coding: utf-8 -*-
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase


class Test(ScorePackageScriptTestCase):

    def test_list_materials(self):
        self.create_score()
        self.create_material('foo')
        self.create_material('bar')
        self.create_material('baz')
        self.create_material('quux')
        script = commandlinetools.ManageMaterialScript()
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 2
        self.compare_captured_output(r'''
        Available materials:
            Markup:
                bar [Markup]
                baz [Markup]
                foo [Markup]
                quux [Markup]
        ''')

    def test_list_materials_no_materials(self):
        self.create_score()
        script = commandlinetools.ManageMaterialScript()
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 2
        self.compare_captured_output(r'''
        Available materials:
            No materials available.
        ''')



# -*- coding: utf-8 -*-
import copy
import abjad
from abjad.tools import scoretools
from abjad.tools import systemtools
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate


def add_string_music_to_score(score):
    r'''Adds string music to score.
    '''

    # generate some pitch and rhythm information
    pitch_contour_reservoir = \
        abjad.demos.part.create_pitch_contour_reservoir()
    shadowed_contour_reservoir = \
        abjad.demos.part.shadow_pitch_contour_reservoir(
        pitch_contour_reservoir)
    durated_reservoir = abjad.demos.part.durate_pitch_contour_reservoir(
        shadowed_contour_reservoir)

    # add six dotted-whole notes and the durated contours to each string voice
    for instrument_name, descents in durated_reservoir.items():
        instrument_voice = score['%s Voice' % instrument_name]
        instrument_voice.extend("R1. R1. R1. R1. R1. R1.")
        for descent in descents:
            instrument_voice.extend(descent)

    # apply instrument-specific edits
    abjad.demos.part.edit_first_violin_voice(score, durated_reservoir)
    abjad.demos.part.edit_second_violin_voice(score, durated_reservoir)
    abjad.demos.part.edit_viola_voice(score, durated_reservoir)
    abjad.demos.part.edit_cello_voice(score, durated_reservoir)
    abjad.demos.part.edit_bass_voice(score, durated_reservoir)

    # chop all string parts into 6/4 measures
    strings_staff_group = score['Strings Staff Group']
    with systemtools.ForbidUpdate(score):
        for voice in  iterate(strings_staff_group).by_class(scoretools.Voice):
            shards = mutate(voice[:]).split([(6, 4)], cyclic=True)
            for shard in shards:
                scoretools.Measure((6, 4), shard)


# -*- coding: utf-8 -*-


def all_are_nonnegative_integers(argument):
    '''Is true when `argument` is an iterable collection of nonnegative
    integers. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.all_are_nonnegative_integers([0, 1, 2, 99])
            True

        ::

            >>> mathtools.all_are_nonnegative_integers([0, 1, 2, -99])
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(mathtools.is_nonnegative_integer(_) for _ in argument)
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Beam__is_beamable_01():
    r'''Eighth notes are beamable.
    Quarter notes are not beamable.
    '''

    assert Beam._is_beamable(Note(0, (1, 8)))
    assert not Beam._is_beamable(Note("c'4"))


def test_spannertools_Beam__is_beamable_02():
    r'''Containers are not beamable.
    '''

    assert not Beam._is_beamable(Staff([]))



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection_is_well_formed_01():
    r'''Well-formedness checking runs correctly against leaves.
    '''
    note = Note("c'4")
    assert inspect_(note).is_well_formed()


def test_selectiontools_Selection_is_well_formed_02():
    r'''Well-formedness checking runs correctly against containers.
    '''
    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools.spannertools.ComplexBeam import ComplexBeam


class MeasuredComplexBeam(ComplexBeam):
    r'''Measured complex beam.

    ..  container:: example

        ::

            >>> staff = Staff()
            >>> staff.append(Measure((2, 16), "c'16 d'16"))
            >>> staff.append(Measure((2, 16), "e'16 f'16"))
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ::

            >>> beam = spannertools.MeasuredComplexBeam()
            >>> selector = select().by_leaf(flatten=True)
            >>> leaves = selector(staff)
            >>> attach(beam, leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                {
                    \time 2/16
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    c'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #1
                    d'16
                }
                {
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #2
                    e'16
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    f'16 ]
                }
            }

    Beams leaves in spanner explicitly.

    Groups leaves by measures.

    Formats top-level `span_beam_count` beam between measures.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_span_beam_count',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        isolated_nib_direction=False,
        overrides=None,
        span_beam_count=1,
        ):
        ComplexBeam.__init__(
            self,
            direction=direction,
            isolated_nib_direction=isolated_nib_direction,
            overrides=overrides,
            )
        assert isinstance(span_beam_count, (int, type(None)))
        self._span_beam_count = span_beam_count

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        ComplexBeam._copy_keyword_args(self, new)
        new._span_beam_count = self.span_beam_count

    def _format_before_leaf(self, leaf):
        result = []
        left, right = None, None
        #if leaf.beam.beamable:
        if self._is_beamable(leaf):
            if self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            elif leaf._get_parentage(include_self=False).get_first(
                scoretools.Measure) is not None:
                measure = leaf._get_parentage(include_self=False).get_first(
                    scoretools.Measure)
                # leaf at beginning of measure
                if measure._is_one_of_my_first_leaves(leaf):
                    assert isinstance(self.span_beam_count, int)
                    left = self.span_beam_count
                    right = leaf.written_duration.flag_count
                # leaf at end of measure
                elif measure._is_one_of_my_last_leaves(leaf):
                    assert isinstance(self.span_beam_count, int)
                    left = leaf.written_duration.flag_count
                    right = self.span_beam_count
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                string = r'\set stemLeftBeamCount = #{}'.format(left)
                result.append(string)
            if right is not None:
                string = r'\set stemRightBeamCount = #{}'.format(right)
                result.append(string)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def span_beam_count(self):
        r'''Gets number of span beams between adjacent measures.

        ..  container:: example

            Use one span beam between measures:

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((2, 32), "c'32 d'32"))
                >>> staff.append(Measure((2, 32), "e'32 f'32"))
                >>> selector = select().by_leaf(flatten=True)
                >>> leaves = selector(staff)
                >>> beam = spannertools.MeasuredComplexBeam(span_beam_count=1)
                >>> attach(beam, leaves)
                >>> show(staff) # doctest: +SKIP

            ::

                >>> beam.span_beam_count
                1

        ..  container:: example

            Use two span beams between measures:

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((2, 32), "c'32 d'32"))
                >>> staff.append(Measure((2, 32), "e'32 f'32"))
                >>> beam = spannertools.MeasuredComplexBeam(span_beam_count=2)
                >>> selector = select().by_leaf(flatten=True)
                >>> leaves = selector(staff)
                >>> attach(beam, leaves)
                >>> show(staff) # doctest: +SKIP

            ::

                >>> beam.span_beam_count
                2

        Returns nonnegative integer or none.
        '''
        return self._span_beam_count



# -*- coding: utf-8 -*-
import shutil
import sys
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib


class ScorePackageScriptTestCase(systemtools.TestCase):
    r'''A base test class for score-package scripts.
    '''

    test_path = pathlib.Path(__file__).parent
    score_path = test_path.joinpath('test_score')
    build_path = score_path.joinpath('test_score', 'build')
    distribution_path = score_path.joinpath('test_score', 'distribution')
    materials_path = score_path.joinpath('test_score', 'materials')
    segments_path = score_path.joinpath('test_score', 'segments')
    tools_path = score_path.joinpath('test_score', 'tools')
    fancy_parts_code = stringtools.normalize(r"""
    \book {
        \bookOutputSuffix "cello"
        \score {
            \keepWithTag #'(time cello)
            \include "../segments.ily"
        }
    }
    \book {
        \bookOutputSuffix "viola"
        \score {
            \keepWithTag #'(viola)
            \include "../segments.ily"
        }
    }
    \book {
        \bookOutputSuffix "violin-i"
        \score {
            \keepWithTag #'(first-violin)
            \include "../segments.ily"
        }
    }
    \book {
        \bookOutputSuffix "violin-ii"
        \score {
            \keepWithTag #'(second-violin)
            \include "../segments.ily"
        }
    }
    """)
    fancy_segment_maker_code = stringtools.normalize(r"""
        # -*- coding: utf-8 -*-
        from abjad import attach
        from abjad import iterate
        from abjad import set_
        from abjad.tools import abctools
        from abjad.tools import indicatortools
        from abjad.tools import lilypondfiletools
        from abjad.tools import scoretools
        from abjad.tools import templatetools


        class SegmentMaker(abctools.AbjadObject):

            ### INITIALIZER ###

            def __init__(self, measure_count=1):
                measure_count = int(measure_count)
                assert 0 < measure_count
                self.measure_count = measure_count
                self.score_template = templatetools.StringQuartetScoreTemplate()

            ### SPECIAL METHODS ###

            def __call__(
                self,
                segment_metadata=None,
                previous_segment_metadata=None,
                ):
                score = self.score_template()
                for i in range(self.measure_count):
                    for voice in iterate(score).by_class(scoretools.Voice):
                        measure = scoretools.Measure((4, 4), "c'1")
                        voice.append(measure)
                lilypond_file = lilypondfiletools.LilyPondFile.new(
                    score,
                    includes=['../../stylesheets/stylesheet.ily'],
                    )
                first_bar_number = segment_metadata.get('first_bar_number', 1)
                if 1 < first_bar_number:
                    set_(score).current_bar_number = first_bar_number
                segment_number = segment_metadata.get('segment_number', 1)
                segment_count = segment_metadata.get('segment_count', 1)
                if 1 < segment_number:
                    rehearsal_mark = indicatortools.RehearsalMark()
                    for voice in iterate(score).by_class(scoretools.Voice):
                        for leaf in iterate(voice).by_leaf():
                            attach(rehearsal_mark, leaf)
                            break
                if segment_count <= segment_number:
                    score.add_final_bar_line(
                        abbreviation='|.',
                        to_each_voice=True,
                        )
                segment_metadata['measure_count'] = self.measure_count
                return lilypond_file, segment_metadata
    """)

    ### TEST LIFECYCLE ###

    def setUp(self):
        super(ScorePackageScriptTestCase, self).setUp()
        if self.score_path.exists():
            shutil.rmtree(str(self.score_path))
        self.directory_items = set(self.test_path.iterdir())
        sys.path.insert(0, str(self.score_path))

    def tearDown(self):
        super(ScorePackageScriptTestCase, self).tearDown()
        for path in sorted(self.test_path.iterdir()):
            if path in self.directory_items:
                continue
            if path.is_file():
                path.unlink()
            else:
                shutil.rmtree(str(path))
        sys.path.remove(str(self.score_path))
        for path, module in tuple(sys.modules.items()):
            if not path or not module:
                continue
            if path.startswith('test_score'):
                del(sys.modules[path])

    ### UTILITY METHODS ###

    def collect_segments(self):
        script = commandlinetools.ManageSegmentScript()
        command = ['--collect']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            script(command)

    def create_build_target(
        self,
        force=False,
        expect_error=False,
        ):
        script = commandlinetools.ManageBuildTargetScript()
        command = ['--new']
        if force:
            command.insert(0, '-f')
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            if expect_error:
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
            else:
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        return self.build_path.joinpath('letter-portrait')

    def create_material(
        self,
        material_name='test_material',
        force=False,
        expect_error=False,
        ):
        script = commandlinetools.ManageMaterialScript()
        command = ['--new', material_name]
        if force:
            command.insert(0, '-f')
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            if expect_error:
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
            else:
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        return self.score_path.joinpath(
            self.score_path.name,
            'materials',
            material_name,
            )

    def create_score(self, force=False, expect_error=False):
        script = commandlinetools.ManageScoreScript()
        command = [
            '--new',
            'Test Score',
            '-y', '2016',
            '-n', 'Josiah Wolf Oberholtzer',
            '-e', 'josiah.oberholtzer@gmail.com',
            '-g', 'josiah-wolf-oberholtzer',
            '-l', 'consort',
            '-w', 'www.josiahwolfoberholtzer.com',
            ]
        if force:
            command.insert(0, '-f')
        with systemtools.TemporaryDirectoryChange(str(self.test_path)):
            if expect_error:
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
            else:
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')

    def create_segment(
        self,
        segment_name='test_segment',
        force=False,
        expect_error=False,
        ):
        script = commandlinetools.ManageSegmentScript()
        command = ['--new', segment_name]
        if force:
            command.insert(0, '-f')
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            if expect_error:
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
            else:
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        return self.score_path.joinpath(
            self.score_path.name,
            'segments',
            segment_name,
            )

    def illustrate_material(self, material_name):
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', material_name]
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit as e:
                raise RuntimeError('SystemExit: {}'.format(e.code))

    def illustrate_segment(self, segment_name):
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', segment_name]
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit as e:
                raise RuntimeError('SystemExit: {}'.format(e.code))

    def illustrate_segments(self):
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', '*']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            script(command)

    def install_fancy_segment_maker(self):
        segment_maker_path = self.tools_path.joinpath('SegmentMaker.py')
        with open(str(segment_maker_path), 'w') as file_pointer:
            file_pointer.write(self.fancy_segment_maker_code)
        parts_path = self.build_path.joinpath('parts.ily')
        with open(str(parts_path), 'w') as file_pointer:
            file_pointer.write(self.fancy_parts_code)



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Slur_01():
    r'''Successful slurs, showing single leaf overlap.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    slur = Slur()
    attach(slur, target[2:])
    slur = Slur()
    attach(slur, target[:3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 (
            c'4
            c'4 ) (
            c'4 )
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Slur_02():
    r'''Swapped start and stop.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    slur = Slur()
    attach(slur, target[2:])
    slur = Slur()
    attach(slur, target[:3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 (
            c'4
            c'4 ) (
            c'4 )
        }
        '''
        )

    string = r"\relative c' { c ( c c () c ) }"

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Slur_03():
    r'''Single leaf.
    '''

    string = '{ c () c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Slur_04():
    r'''Unterminated.
    '''

    string = '{ c ( c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Slur_05():
    r'''Unstarted.
    '''

    string = '{ c c c c ) }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Slur_06():
    r'''Nested.
    '''

    string = '{ c ( c ( c ) c ) }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Slur_07():
    r'''With direction.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    slur = Slur(direction=Down)
    attach(slur, target[:3])
    slur = Slur(direction=Up)
    attach(slur, target[2:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 _ (
            c'4
            c'4 ) ^ (
            c'4 )
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Registration___contains___01():
    r'''Works with components.
    '''

    mapping = pitchtools.Registration([('[A0, C4)', 15), ('[C4, C8)', 27)])
    assert pitchtools.RegistrationComponent('[A0, C4)', 15) in mapping


def test_pitchtools_Registration___contains___02():
    r'''Works with component items.
    '''

    mapping = pitchtools.Registration([('[A0, C4)', 15), ('[C4, C8)', 27)])
    assert ('[A0, C4)', 15) in mapping



# -*- coding: utf-8 -*-
from abjad.tools.scoretools.Context import Context


class Voice(Context):
    r'''A musical voice.

    ::

        >>> voice = Voice("c'8 d'8 e'8 f'8")

    ::

        >>> voice
        Voice("c'8 d'8 e'8 f'8")

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(voice) # doctest: +SKIP

    Returns voice instance.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = ()

    _default_context_name = 'Voice'

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Voice', name=None):
        Context.__init__(
            self,
            music=music,
            context_name=context_name,
            name=name,
            )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension_is_empty_01():

    chord_suspension = tonalanalysistools.ChordSuspension()

    assert chord_suspension.is_empty



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_FixedDurationTuplet_force_fraction_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    tuplet.force_fraction = True

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class Hairpin(Spanner):
    r'''Hairpin.

    ..  container:: example

        Crescendo:

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='p < f',
            ...     include_rests=False,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4
                c'8 \< \p
                d'8
                e'8
                f'8 \f
                r4
            }

        Decrescendo:

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='f > p',
            ...     include_rests=False,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4
                c'8 \> \f
                d'8
                e'8
                f'8 \p
                r4
            }

    ..  container:: example

        Crescendo dal niente:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='niente < f',
            ...     include_rests=False,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \once \override Hairpin.circled-tip = ##t
                c'4 \<
                d'4
                e'4
                f'4 \f
            }

        Decrescendo al niente:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='f > niente',
            ...     include_rests=False,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \once \override Hairpin.circled-tip = ##t
                c'4 \> \f
                d'4
                e'4
                f'4 \!
            }

    .. todo:: Make niente hairpins work with ``include_rests=True``.
    '''

    ### CLASS VARIABLES ###

    _hairpin_shape_strings = (
        '<',
        '>',
        )

    __slots__ = (
        '_descriptor',
        '_direction',
        '_include_rests',
        '_shape_string',
        '_start_dynamic',
        '_stop_dynamic',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        descriptor='<',
        direction=None,
        include_rests=False,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction
        self._include_rests = include_rests
        assert self._is_valid_descriptor(descriptor), repr(descriptor)
        start_dynamic, shape_string, stop_dynamic = self._parse_descriptor(
            descriptor)
        self._descriptor = descriptor
        assert shape_string in ('<', '>')
        self._shape_string = shape_string
        if start_dynamic is not None:
            start_dynamic = indicatortools.Dynamic(start_dynamic)
        self._start_dynamic = start_dynamic
        if stop_dynamic is not None:
            stop_dynamic = indicatortools.Dynamic(stop_dynamic)
        self._stop_dynamic = stop_dynamic

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, component_expression):
        if isinstance(component_expression, scoretools.Leaf):
            return False
        if not self._at_least_two_leaves(component_expression):
            return False
        formattable_components = []
        for component in component_expression:
            if (self.include_rests or
                isinstance(component, (scoretools.Note, scoretools.Chord))):
                formattable_components.append(component)
        return 1 < len(formattable_components)

    def _copy_keyword_args(self, new):
        Spanner._copy_keyword_args(self, new)
        new._direction = self.direction
        new._include_rests = self.include_rests
        new._shape_string = self.shape_string
        new._start_dynamic = self.start_dynamic
        new._stop_dynamic = self.stop_dynamic

    def _format_time_test(self, leaf):
        if not 1 < len(self._get_leaves()):
            message = '{} fails format-time test.'
            message = message.format(self)
            raise Exception(message)

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        self._format_time_test(leaf)
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
        direction_string = ''
        if self.direction is not None:
            direction_string = \
                stringtools.to_tridirectional_lilypond_symbol(
                    self.direction)
            direction_string = '{} '.format(direction_string)
        if (self._is_my_first_leaf(leaf) and
            (self.start_dynamic and self.start_dynamic.name == 'niente' or
            self.stop_dynamic and self.stop_dynamic.name == 'niente')):
            override(leaf).hairpin.circled_tip = True
        if self.include_rests:
            if self._is_my_first_leaf(leaf):
                string = r'{}\{}'.format(direction_string, self.shape_string)
                lilypond_format_bundle.right.spanner_starts.append(string)
                if (self.start_dynamic and
                    not self.start_dynamic.name == 'niente'):
                        string = r'{}\{}'.format(
                            direction_string,
                            self.start_dynamic.name,
                            )
                        lilypond_format_bundle.right.spanner_starts.append(
                            string)
            if self._is_my_last_leaf(leaf):
                if (self.stop_dynamic and
                    not self.stop_dynamic.name == 'niente'):
                        string = r'{}\{}'.format(
                            direction_string,
                            self.stop_dynamic.name,
                            )
                        lilypond_format_bundle.right.spanner_stops.append(
                            string)
                else:
                    effective_dynamic = leaf._get_effective(
                        indicatortools.Dynamic)
                    if (effective_dynamic is None or
                        effective_dynamic.name == 'niente'):
                        string = r'\!'
                        lilypond_format_bundle.right.spanner_stops.append(
                            string)
                    elif effective_dynamic not in leaf._indicator_expressions:
                        found_match = False
                        for indicator in leaf._get_indicators(
                            indicatortools.Dynamic):
                            if indicator == effective_dynamic:
                                found_match = True
                        if not found_match:
                            string = r'\!'
                            lilypond_format_bundle.right.spanner_stops.append(
                                string)
        else:
            if self._is_my_first(leaf, (scoretools.Chord, scoretools.Note)):
                string = r'{}\{}'.format(
                    direction_string,
                    self.shape_string,
                    )
                lilypond_format_bundle.right.spanner_starts.append(string)
                if (self.start_dynamic and
                    not self.start_dynamic.name == 'niente'):
                        string = r'{}\{}'.format(
                            direction_string,
                            self.start_dynamic.name,
                            )
                        lilypond_format_bundle.right.spanner_starts.append(
                            string)
            if self._is_my_last(leaf, (scoretools.Chord, scoretools.Note)):
                if (self.stop_dynamic and
                    not self.stop_dynamic.name == 'niente'):
                        string = r'{}\{}'.format(
                            direction_string,
                            self.stop_dynamic.name,
                            )
                        lilypond_format_bundle.right.spanner_stops.append(
                            string)
                else:
                    effective_dynamic = leaf._get_effective(
                        indicatortools.Dynamic)
                    if (effective_dynamic is None or
                        effective_dynamic.name == 'niente'):
                        string = r'\!'
                        lilypond_format_bundle.right.spanner_stops.append(
                            string)
                    elif effective_dynamic not in leaf._indicator_expressions:
                        found_match = False
                        for indicator in leaf._get_indicators(
                            indicatortools.Dynamic):
                            if indicator == effective_dynamic:
                                found_match = True
                        if not found_match:
                            string = r'\!'
                            lilypond_format_bundle.right.spanner_stops.append(
                                string)
        if self._is_my_only_leaf(leaf):
            lilypond_format_bundle.right.spanner_starts.extend(
                lilypond_format_bundle.right.spanner_stops)
            lilypond_format_bundle.right.spanner_stops[:] = []
        return lilypond_format_bundle

    @staticmethod
    def _is_hairpin_shape_string(arg):
        return arg in Hairpin._hairpin_shape_strings

    @staticmethod
    def _is_hairpin_token(arg):
        r'''Is true when `arg` is a hairpin token. Otherwise false:

        ::

            >>> spannertools.Hairpin._is_hairpin_token(('p', '<', 'f'))
            True

        ::

            >>> spannertools.Hairpin._is_hairpin_token(('f', '<', 'p'))
            False

        Returns true or false.
        '''
        Dynamic = indicatortools.Dynamic
        if (isinstance(arg, tuple) and
            len(arg) == 3 and
            (not arg[0] or indicatortools.Dynamic.is_dynamic_name(arg[0]))
            and Hairpin._is_hairpin_shape_string(arg[1]) and
            (not arg[2] or indicatortools.Dynamic.is_dynamic_name(arg[2]))):
            if arg[0] and arg[2]:
                start_ordinal = Dynamic.dynamic_name_to_dynamic_ordinal(arg[0])
                stop_ordinal = Dynamic.dynamic_name_to_dynamic_ordinal(arg[2])
                if arg[1] == '<':
                    return start_ordinal < stop_ordinal
                else:
                    return stop_ordinal < start_ordinal
            else:
                return True
        else:
            return False

    def _is_valid_descriptor(self, descriptor):
        start, shape, stop = self._parse_descriptor(descriptor)
        if shape not in self._hairpin_shape_strings:
            return False
        if (start is not None and
            start not in indicatortools.Dynamic._dynamic_names):
            return False
        if (stop is not None and
            stop not in indicatortools.Dynamic._dynamic_names):
            return False
        if start is not None and stop is not None:
            start_ordinal = \
                indicatortools.Dynamic.dynamic_name_to_dynamic_ordinal(start)
            stop_ordinal = \
                indicatortools.Dynamic.dynamic_name_to_dynamic_ordinal(stop)
            if shape == '<' and not start_ordinal < stop_ordinal:
                return False
            if shape == '>' and not start_ordinal > stop_ordinal:
                return False
        return True

    def _parse_descriptor(self, descriptor):
        r'''Example descriptors:

        ::

            '<'
            'p <'
            'p < f'

        '''
        assert isinstance(descriptor, str)
        parts = descriptor.split()
        num_parts = len(parts)
        start, shape, stop = None, None, None
        if parts[0] in ('<', '>'):
            assert 1 <= num_parts <= 2
            if num_parts == 1:
                shape = parts[0]
            else:
                shape = parts[0]
                stop = parts[1]
        else:
            assert 2 <= num_parts <= 3
            if num_parts == 2:
                start = parts[0]
                shape = parts[1]
            else:
                start = parts[0]
                shape = parts[1]
                stop = parts[2]
        assert shape in ('<', '>')
        return start, shape, stop

    ### PUBLIC PROPERTIES ###

    @property
    def descriptor(self):
        r'''Gets descriptor of hairpin.

        ..  container:: example

            Gets descriptor of crescendo:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> hairpin.descriptor
                'p < f'

        Returns string.
        '''
        return self._descriptor

    @property
    def direction(self):
        r'''Gets direction.

        ..  container:: example

            Positions hairpin above staff:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(
                ...     descriptor='p < f',
                ...     direction=Up,
                ...     )
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4
                c'8 ^ \< ^ \p
                d'8
                e'8
                f'8 ^ \f
                r4
            }

            ::

                >>> hairpin.direction
                '^'

        Defaults to none.

        Set to up, down or none.

        Returns up, down or none.
        '''
        return self._direction

    @property
    def include_rests(self):
        r'''Gets include-rests flag of hairpin.

        ..  container:: example

            Crescendo includes rests:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(
                ...     descriptor='p < f',
                ...     include_rests=True,
                ...     )
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4 \< \p
                c'8
                d'8
                e'8
                f'8
                r4 \f
            }

            ::

                >>> hairpin.include_rests
                True

        Returns true or false.
        '''
        return self._include_rests

    @property
    def shape_string(self):
        r'''Gets shape string of hairpin.

        ..  container:: example

            Gets shape string of crescendo:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> hairpin.shape_string
                '<'

        Returns string.
        '''
        return self._shape_string

    @property
    def start_dynamic(self):
        r'''Gets start dynamic string of hairpin.

        ..  container:: example

            Gets start dynamic of crescendo:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> hairpin.start_dynamic
                Dynamic(name='p')

        Returns dynamic or none.
        '''
        return self._start_dynamic

    @property
    def stop_dynamic(self):
        r'''Gets stop dynamic string of hairpin.

        ..  container:: example

            Gets stop dynamic of crescendo:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> hairpin.stop_dynamic
                Dynamic(name='f')

        Returns dynamic or none.
        '''
        return self._stop_dynamic



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondfiletools
from abjad.tools import markuptools
from abjad.tools import mathtools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import stringtools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import set_


class GalleryMaker(AbjadValueObject):
    r'''Gallery maker.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Illustration helpers'

    ### SPECIAL METHODS ###

    def __call__(self, configurations_by_class):
        r'''Calls gallery-maker.

        Returns LilyPond file.
        '''
        lilypond_file = lilypondfiletools.LilyPondFile.new()
        lilypond_file.items.remove(lilypond_file.header_block)
        lilypond_file.items.remove(lilypond_file.layout_block)
        lilypond_file.items.remove(lilypond_file.paper_block)
        lilypond_file.items.remove(lilypond_file.score_block)
        for class_ in configurations_by_class:
            configurations = configurations_by_class[class_]
            for i, configuration in enumerate(configurations):
                configuration_number = i + 1
                score_blocks = self._configuration_to_score_blocks(
                    configuration,
                    configuration_number,
                    )
                first_score_block = score_blocks[0]
                lilypond_file.items.extend(score_blocks)
                string = r'\pageBreak'
                lilypond_file.items.append(string)
        assert lilypond_file.items[-1] == string
        lilypond_file.items.pop(-1)
        lilypond_file.includes.append('stylesheet.ily')
        return lilypond_file

    ### PRIVATE METHODS ###

    def _add_class_name_header(self, rhythm_maker, score_block):
        header_block = lilypondfiletools.Block(name='header')
        class_name_markup = self._make_class_name_markup(rhythm_maker)
        header_block.title = class_name_markup
        score_block.items.append(header_block)

    @staticmethod
    def _add_final_bar_line(score):
        score.add_final_bar_line()
        selector = select().by_leaf(flatten=True)
        leaves = selector(score)
        leaves = leaves[-1:]
        last_leaf = leaves[0]
        string = "override Staff.BarLine.extra-offset = #'(1.6 . 0)"
        command = indicatortools.LilyPondCommand(
            string,
            'after',
            )
        attach(command, last_leaf)

    def _attach_configuration_markup(
        self,
        rhythm_maker,
        first_score,
        ):
        markup = self._make_configuration_markup(rhythm_maker)
        context = first_score['TimeSignatureContext']
        for leaf in iterate(context).by_leaf():
            break
        first_leaf = leaf
        attach(markup, first_leaf)

    @staticmethod
    def _check_score(score):
        if inspect_(score).is_well_formed():
            return
        violations = inspect_(score).tabulate_well_formedness_violations()
        message = 'score is not well-formed: {!r}.'
        message = message.format(score)
        message += '\n'
        message += violations
        raise Exception(message)

    def _configuration_to_score_blocks(
        self,
        configuration,
        configuration_number,
        ):
        assert isinstance(configuration, tuple) and len(configuration) == 2
        rhythm_maker = configuration[0]
        division_lists = configuration[1]
        score_blocks = []
        for i, division_list in enumerate(division_lists):
            score_number = i + 1
            score_number_markup = self._make_score_number_markup(
                configuration_number,
                score_number,
                )
            score = self._make_score(
                rhythm_maker,
                division_list,
                score_number_markup,
                )
            score_block = lilypondfiletools.Block(name='score')
            score_block.items.append(score)
            score_blocks.append(score_block)
        first_score_block = score_blocks[0]
        first_score = first_score_block.items[0]
        self._attach_configuration_markup(rhythm_maker, first_score)
        if configuration_number == 1:
            self._add_class_name_header(rhythm_maker, first_score_block)
        return score_blocks

    @staticmethod
    def _make_class_name_markup(rhythm_maker):
        class_ = type(rhythm_maker)
        string = class_.__name__
        string = stringtools.capitalize_start(string)
        pair = schemetools.SchemePair('font-name', 'Times')
        command = markuptools.MarkupCommand('override', pair, string)
        command = markuptools.MarkupCommand('fontsize', 4.5, command)
        markup = markuptools.Markup(command)
        return markup

    @staticmethod
    def _make_configuration_markup(rhythm_maker):
        string = format(rhythm_maker, 'storage')
        string = string.replace('rhythmmakertools.', '')
        lines = string.split('\n')
        command = markuptools.MarkupCommand('column', lines)
        command.force_quotes = True
        pair = schemetools.SchemePair('font-name', 'Courier')
        command = markuptools.MarkupCommand('override', pair, command)
        markup = markuptools.Markup(command, direction=Up)
        return markup

    def _make_score(
        self,
        rhythm_maker,
        division_list,
        score_number_markup,
        ):
        lists = rhythm_maker(division_list)
        selections = sequencetools.Sequence(lists).flatten()
        measures = scoretools.make_spacer_skip_measures(division_list)
        time_signature_context = scoretools.Context(
            measures,
            context_name='TimeSignatureContext',
            name='TimeSignatureContext',
            )
        measures = scoretools.make_spacer_skip_measures(division_list)
        staff = scoretools.Staff(measures)
        set_(staff).instrument_name = score_number_markup
        staff.context_name = 'RhythmicStaff'
        staff.name = 'Note-entry staff'
        measures = mutate(staff).replace_measure_contents(selections)
        score = scoretools.Score()
        score.append(time_signature_context)
        score.append(staff)
        self._add_final_bar_line(score)
        self._check_score(score)
        return score

    @staticmethod
    def _make_score_number_markup(
        configuration_number,
        score_number,
        ):
        assert mathtools.is_nonnegative_integer(configuration_number)
        assert mathtools.is_nonnegative_integer(score_number)
        number_string = '{}-{}'.format(
            configuration_number,
            score_number,
            )
        command = markuptools.MarkupCommand('fontsize', 2, number_string)
        command = markuptools.MarkupCommand('italic', command)
        command = markuptools.MarkupCommand('box', command)
        pair = schemetools.SchemePair('box-padding', 0.75)
        command = markuptools.MarkupCommand('override', pair, command)
        width = 9
        command = markuptools.MarkupCommand('hcenter-in', width, command)
        markup = markuptools.Markup(command)
        return markup



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection_partition_by_durations_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )


    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [1.5],
        cyclic=True,
        fill=Exact,
        in_seconds=True,
        overhang=False
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)],
        [Note(g', 8), Note(f', 8), Note(e', 8)]
    ]
    '''

    assert len(parts) == 2
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:3])
    assert parts[1] == list(leaves[3:6])


def test_selectiontools_Selection_partition_by_durations_02():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [1.5],
        cyclic=True,
        fill=Exact,
        in_seconds=True,
        overhang=True,
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)],
        [Note(g', 8), Note(f', 8), Note(e', 8)],
        [Note(d', 8), Note(c', 8)]
    ]
    '''

    assert len(parts) == 3
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:3])
    assert parts[1] == list(leaves[3:6])
    assert parts[2] == list(leaves[6:8])


def test_selectiontools_Selection_partition_by_durations_03():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [Duration(3, 8)],
        cyclic=True,
        fill=Exact,
        in_seconds=False,
        overhang=True,
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)]
        [Note(g', 8), Note(f', 8), Note(e', 8)]
        [Note(d', 8), Note(c', 8)]
    ]
    '''

    assert len(parts) == 3
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:3])
    assert parts[1] == list(leaves[3:6])
    assert parts[2] == list(leaves[6:8])


def test_selectiontools_Selection_partition_by_durations_04():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [1.5],
        cyclic=False,
        fill=Exact,
        in_seconds=True,
        overhang=False,
        )

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(parts) == 1
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:3])


def test_selectiontools_Selection_partition_by_durations_05():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [Duration(3, 8)],
        cyclic=False,
        fill=Exact,
        in_seconds=False,
        overhang=False,
        )

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(parts) == 1
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:3])


def test_selectiontools_Selection_partition_by_durations_06():

    staff = Staff()
    staff.extend("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    staff.extend("abj: | 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [Duration(3, 16), Duration(1, 16)],
        cyclic=True,
        fill=More,
        in_seconds=False,
        overhang=True,
        )

    r'''
    [
        [Note(c', 8), Note(d', 8)],
        [Note(e', 8)], [Note(f', 8), Note(g', 8)],
        [Note(a', 8)], [Note(b', 8), Note(c'', 8)]
    ]
    '''

    assert len(parts) == 5
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:2])
    assert parts[1] == list(leaves[2:3])
    assert parts[2] == list(leaves[3:5])
    assert parts[3] == list(leaves[5:6])
    assert parts[4] == list(leaves[6:])


def test_selectiontools_Selection_partition_by_durations_07():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [0.75],
        cyclic=True,
        fill=Less,
        in_seconds=True,
        overhang=False,
        )

    r'''
    [
        [Note(c', 8)],
        [Note(d', 8)],
        [Note(e', 8)],
        [Note(f', 8)],
        [Note(g', 8)],
        [Note(a', 8)],
        [Note(b', 8)]
    ]
    '''

    assert len(parts) == 7
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:1])
    assert parts[1] == list(leaves[1:2])
    assert parts[2] == list(leaves[2:3])
    assert parts[3] == list(leaves[3:4])
    assert parts[4] == list(leaves[4:5])
    assert parts[5] == list(leaves[5:6])
    assert parts[6] == list(leaves[6:7])


def test_selectiontools_Selection_partition_by_durations_08():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [Duration(3, 16)],
        cyclic=True,
        fill=Less,
        in_seconds=False,
        overhang=False,
        )

    r'''
    [
        [Note(c', 8)],
        [Note(d', 8)],
        [Note(e', 8)],
        [Note(f', 8)],
        [Note(g', 8)],
        [Note(a', 8)],
        [Note(b', 8)]
    ]
    '''

    assert len(parts) == 7
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:1])
    assert parts[1] == list(leaves[1:2])
    assert parts[2] == list(leaves[2:3])
    assert parts[3] == list(leaves[3:4])
    assert parts[4] == list(leaves[4:5])
    assert parts[5] == list(leaves[5:6])
    assert parts[6] == list(leaves[6:7])


def test_selectiontools_Selection_partition_by_durations_09():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [0.75],
        cyclic=False,
        fill=Less,
        in_seconds=True,
        overhang=False,
        )

    "[[Note(c', 8)]]"

    assert len(parts) == 1
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:1])


def test_selectiontools_Selection_partition_by_durations_10():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    parts = leaves.partition_by_durations(
        [Duration(3, 16)],
        cyclic=False,
        fill=Less,
        in_seconds=False,
        overhang=False,
        )

    "[[Note(c', 8)]]"

    assert len(parts) == 1
    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    assert parts[0] == list(leaves[:1])


# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class AttackPointOptimizer(AbjadObject):
    r'''Abstract attack-point optimizer class from which concrete
    attack-point optimizer classes inherit.

    Attack-point optimizers may alter the number, order, and individual
    durations of leaves in a logical tie, but may not alter the overall
    duration of that logical tie.

    They effectively "clean up" notation, post-quantization.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, argument):
        r'''Calls attack-point optimizer.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad.tools import abctools
from docutils import nodes


class CodeOutputProxy(abctools.AbjadValueObject):
    r'''A code output proxy.

    ::

        >>> from abjad.tools import abjadbooktools
        >>> proxy = abjadbooktools.CodeOutputProxy([
        ...     ">>> print('Hello, world!')",
        ...     'Hello, world!',
        ...     '>>> 1 + 1',
        ...     '2',
        ...     ])
        >>> print(format(proxy))
        abjadbooktools.CodeOutputProxy(
            (
                ">>> print('Hello, world!')",
                'Hello, world!',
                '>>> 1 + 1',
                '2',
                )
            )

    ::

        >>> for line in proxy.as_latex():
        ...     line
        ...
        '\\begin{lstlisting}'
        ">>> print('Hello, world!')"
        'Hello, world!'
        '\\end{lstlisting}'
        '\\begin{lstlisting}'
        '>>> 1 + 1'
        '2'
        '\\end{lstlisting}'

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Output Proxies'

    __slots__ = (
        '_code_block_specifier',
        '_payload',
        )

    ### INITIALIZER ###

    def __init__(self, payload, code_block_specifier=None):
        self._payload = tuple(payload)
        self._code_block_specifier = code_block_specifier

    ### PUBLIC METHODS ###

    def as_docutils(
        self,
        configuration=None,
        output_directory=None,
        ):
        r'''Creates a docutils node representation of the code output proxy.

        Returns list of docutils nodes.
        '''
        result = []
        try:
            waiting_for_prompt = False
            lines = []
            for line in self.payload:
                if not line.startswith(('>>> ', '... ')):
                    waiting_for_prompt = True
                elif line.startswith('>>> ') and waiting_for_prompt:
                    waiting_for_prompt = False
                    code = u'\n'.join(lines)
                    block = nodes.literal_block(code, code)
                    result.append(block)
                    lines = []
                lines.append(line)
            if lines:
                code = u'\n'.join(lines)
                block = nodes.literal_block(code, code)
                result.append(block)
        except UnicodeDecodeError:
            print()
            print(type(self))
            for line in self.payload:
                print(repr(line))
        return result

    def as_latex(
        self,
        configuration=None,
        output_directory=None,
        relative_output_directory=None,
        ):
        r'''Creates a LaTeX representation of the code output proxy.

        Returns list of strings.
        '''
        #lexer = 'pycon'
        #if not self.payload[0].startswith('>>>'):
        #    lexer = 'python'

        configuration = configuration or {}
        latex_configuration = configuration.get('latex', {})
        start_command = latex_configuration.get(
            'code-block-start',
            [r'\begin{lstlisting}'],
            )
        start_command = '\n'.join(start_command)
        stop_command = latex_configuration.get(
            'code-block-stop',
            [r'\end{lstlisting}'],
            )
        stop_command = '\n'.join(stop_command)
        result = []
        result.append(start_command)
        waiting_for_prompt = False
        for line in self.payload:
            if not line.startswith(('>>> ', '... ')):
                waiting_for_prompt = True
            elif line.startswith('>>> ') and waiting_for_prompt:
                waiting_for_prompt = False
                result.append(stop_command)
                result.append(start_command)
            result.append(line)
        result.append(stop_command)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def code_block_specifier(self):
        r'''Gets code block specifier.
        '''
        return self._code_block_specifier

    @property
    def payload(self):
        r'''Gets code output proxy payload.

        Returns tuple of strings.
        '''
        return self._payload



# -*- coding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection
from abjad.tools.topleveltools import new


class TypedFrozenset(TypedCollection):
    r'''A typed fozen set.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        items = items or []
        items = [self._item_coercer(_) for _ in items]
        self._collection = frozenset(items)

    ### SPECIAL METHODS ###

    def __and__(self, argument):
        r'''Logical AND of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.__and__(argument._collection)
        result = type(self)(result)
        return result

    def __ge__(self, argument):
        r'''Is true when typed frozen set is greater than or equal to `argument`.
        Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__ge__(argument._collection)

    def __gt__(self, argument):
        r'''Is true when typed frozen set is greater than `argument`. Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__gt__(argument._collection)

    def __hash__(self):
        r'''Hashes typed frozen set.

        Returns integer.
        '''
        return hash((
            type(self),
            self._collection,
            self.item_class,
            ))

    def __le__(self, argument):
        r'''Is true when typed frozen set is less than or equal to `argument`.
        Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__le__(argument._collection)

    def __lt__(self, argument):
        r'''Is true when typed frozen set is less than `argument`. Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__lt__(argument._collection)

    def __ne__(self, argument):
        r'''Is true when typed frozen set is not equal to `argument`. Otherwise
        false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__ne__(argument._collection)

    def __or__(self, argument):
        r'''Logical OR of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.__or__(argument._collection)
        result = type(self)(result)
        return result

    def __sub__(self, argument):
        r'''Subtracts `argument` from typed frozen set.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.__sub__(argument._collection)
        result = type(self)(result)
        return result

    def __xor__(self, argument):
        r'''Logical XOR of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.__xor__(argument._collection)
        result = type(self)(result)
        return result

    ### PUBLIC METHODS ###

    def copy(self):
        r'''Copies typed frozen set.

        Returns new typed frozen set.
        '''
        return type(self)(self._collection.copy())

    def difference(self, argument):
        r'''Typed frozen set set-minus `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.difference(argument._collection)
        result = type(self)(result)
        return result

    def intersection(self, argument):
        r'''Set-theoretic intersection of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.intersection(argument._collection)
        result = type(self)(result)
        return result

    def isdisjoint(self, argument):
        r'''Is true when typed frozen set shares no elements with `argument`.
        Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.isdisjoint(argument._collection)

    def issubset(self, argument):
        r'''Is true when typed frozen set is a subset of `argument`. Otherwise
        false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.issubset(argument._collection)

    def issuperset(self, argument):
        r'''Is true when typed frozen set is a superset of `argument`. Otherwise
        false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.issuperset(argument._collection)

    def symmetric_difference(self, argument):
        r'''Symmetric difference of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.symmetric_difference(argument._collection)
        result = type(self)(result)
        return result

    def union(self, argument):
        r'''Union of typed frozen set and `argument`.

        Returns new typed frozen set.
        '''
        argument = type(self)(argument)
        result = self._collection.union(argument._collection)
        result = type(self)(result)
        return result


collections.Set.register(TypedFrozenset)



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.Heuristic import Heuristic


class DistanceHeuristic(Heuristic):
    r'''Concrete ``Heuristic`` subclass which considers only the
    computed distance of each ``QGrid`` and the number of leaves of
    that ``QGrid`` when choosing the optimal ``QGrid`` for a given
    ``QTargetBeat``.

    The ``QGrid`` with the smallest distance and fewest number of
    leaves will be selected.

    Return ``DistanceHeuristic`` instance.
    '''

    ### PRIVATE METHODS ###

    def _process(self, q_target_beats):
        from abjad.tools import quantizationtools
        for q_target_beat in q_target_beats:
            q_grids = q_target_beat.q_grids
            if q_grids:
                sorted_q_grids = sorted(
                    q_grids, key=lambda x: (x.distance, len(x.leaves)))
                q_target_beat._q_grid = sorted_q_grids[0]
            else:
                q_target_beat._q_grid = quantizationtools.QGrid()
        return q_target_beats



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_respell_with_sharps_01():

    note = Note(('df', 4), 4)
    mutate(note).respell_with_sharps()

    assert note.written_pitch == NamedPitch('cs', 4)


def test_agenttools_MutationAgent_respell_with_sharps_02():

    chord = Chord([('df', 4), ('f', 4), ('af', 4)], (1, 4))
    mutate(chord).respell_with_sharps()

    assert chord.written_pitches == (
        NamedPitch('cs', 4),
        NamedPitch('f', 4),
        NamedPitch('gs', 4),
        )


def test_agenttools_MutationAgent_respell_with_sharps_03():

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8 af'8 a'8 bf'8 b'")
    mutate(staff).respell_with_sharps()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ds'8
            e'8
            f'8
            fs'8
            g'8
            gs'8
            a'8
            as'8
            b'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RootedChordClass_root_string_01():

    assert tonalanalysistools.RootedChordClass('c', 'major', 'triad').root_string == 'C'
    assert tonalanalysistools.RootedChordClass('c', 'minor', 'triad').root_string == 'c'
    assert tonalanalysistools.RootedChordClass('cs', 'major', 'triad').root_string == 'C#'
    assert tonalanalysistools.RootedChordClass('cs', 'minor', 'triad').root_string == 'c#'
    assert tonalanalysistools.RootedChordClass('cf', 'major', 'triad').root_string == 'Cb'
    assert tonalanalysistools.RootedChordClass('cf', 'minor', 'triad').root_string == 'cb'



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class LilyPondLanguageToken(AbjadValueObject):
    r'''A LilyPond file ``\language`` token.

    ..  container:: example

        ::

            >>> lilypondfiletools.LilyPondLanguageToken()
            LilyPondLanguageToken()

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond language token.

        ..  container:: example

            ::

                >>> token = lilypondfiletools.LilyPondLanguageToken()
                >>> print(format(token))
                \language "english"

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond language token.

        ..  container:: example

            ::

                >>> token = lilypondfiletools.LilyPondLanguageToken()
                >>> token
                LilyPondLanguageToken()

        Returns string.
        '''
        return '{}()'.format(type(self).__name__)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        string = r'\language "english"'
        return string



# -*- coding: utf-8 -*-
import numbers


def is_integer_equivalent_number(argument):
    '''Is true when `argument` is a number and `argument` is equivalent to an
    integer. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_equivalent_number(12.0)
            True

        ::

            >>> mathtools.is_integer_equivalent_number(Duration(1, 2))
            False

    Returns true or false.
    '''
    if isinstance(argument, numbers.Number):
        if int(argument) == argument:
            return True
    return False



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Annotation___setattr___01():
    r'''Slots constrain annotation attributes.
    '''

    annotation = indicatortools.Annotation('foo')

    assert pytest.raises(AttributeError, "annotation.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Note_grace_01():
    r'''Attach one grace note.
    '''

    note = Note("c'4")
    grace_container = scoretools.GraceContainer([Note(2, (1, 16))])
    attach(grace_container, note)

    assert format(note) == stringtools.normalize(
        r'''
        \grace {
            d'16
        }
        c'4
        '''
        )


def test_scoretools_Note_grace_02():
    r'''Attach several grace notes.
    '''

    note = Note("c'4")
    grace_container = scoretools.GraceContainer([Note(0, (1, 16)), Note(2, (1, 16)), Note(4, (1, 16))])
    attach(grace_container, note)

    assert format(note) == stringtools.normalize(
        r'''
        \grace {
            c'16
            d'16
            e'16
        }
        c'4
        '''
        )


def test_scoretools_Note_grace_03():
    r'''Attach one appoggiatura.
    '''

    note = Note("c'4")
    grace_container = scoretools.GraceContainer([Note(2, (1, 16))], kind='appoggiatura')
    attach(grace_container, note)

    assert format(note) == stringtools.normalize(
        r'''
        \appoggiatura {
            d'16
        }
        c'4
        '''
        )


def test_scoretools_Note_grace_04():
    r'''Attach one acciaccatura.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(2, (1, 16))], kind='acciaccatura')
    attach(grace, note)

    assert format(note) == stringtools.normalize(
        r'''
        \acciaccatura {
            d'16
        }
        c'4
        '''
        )


def test_scoretools_Note_grace_05():
    r'''Attach one after grace note.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(2, (1, 16))], kind='after')
    attach(grace, note)

    assert format(note) == stringtools.normalize(
        r'''
        \afterGrace
        c'4
        {
            d'16
        }
        '''
        )


def test_scoretools_Note_grace_06():
    r'''Attach several after grace notes.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(0, (1, 16)), Note(2, (1, 16)), Note(4, (1, 16))], kind='after')
    attach(grace, note)

    assert format(note) == stringtools.normalize(
        r'''
        \afterGrace
        c'4
        {
            c'16
            d'16
            e'16
        }
        '''
        )



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class GraceHandler(AbjadObject):
    r'''Abstract base class from which concrete ``GraceHandler``
    subclasses inherit.

    Determines what pitch, if any, will be selected from a list of
    ``QEvents`` to be applied to an attack-point generated by a ``QGrid``,
    and whether there should be a ``GraceContainer`` attached to th
    at attack-point.

    When called on a sequence of ``QEvents``, ``GraceHandler``
    subclasses should return a pair, where the first item of the pair
    is a sequence of pitch tokens or ``None``, and where the
    second item of the pair is a ``GraceContainer`` instance or None.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, q_events):
        r'''Calls grace handler.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_effective_staff_01():
    r'''Staff changes work on the first note of a staff.
    '''

    staves = 2 * Staff("c'8 d'8 e'8 f'8")
    staff_group = StaffGroup(staves)
    staff_group.context_name = 'PianoStaff'
    staff_group.is_simultaneous = True
    staff_group[0].name = 'RH'
    staff_group[1].name = 'LH'
    staff_change = indicatortools.StaffChange(staff_group[1])
    attach(staff_change, staff_group[0][0])

    assert format(staff_group) == stringtools.normalize(
        r'''
        \new PianoStaff <<
            \context Staff = "RH" {
                \change Staff = LH
                c'8
                d'8
                e'8
                f'8
            }
            \context Staff = "LH" {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(staff_group).is_well_formed()
    assert inspect_(staff_group[0][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][2]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][3]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][2]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][3]).get_effective_staff() is staff_group[1]


def test_agenttools_InspectionAgent_get_effective_staff_02():
    r'''Staff changes work on middle notes of a staff.
    '''

    staves = 2 * Staff("c'8 d'8 e'8 f'8")
    staff_group = StaffGroup(staves)
    staff_group.context_name = 'PianoStaff'
    staff_group.is_simultaneous = True
    staff_group[0].name = 'RH'
    staff_group[1].name = 'LH'
    staff_change = indicatortools.StaffChange(staff_group[1])
    attach(staff_change, staff_group[0][0])
    staff_change = indicatortools.StaffChange(staff_group[0])
    attach(staff_change, staff_group[0][2])

    assert format(staff_group) == stringtools.normalize(
        r'''
        \new PianoStaff <<
            \context Staff = "RH" {
                \change Staff = LH
                c'8
                d'8
                \change Staff = RH
                e'8
                f'8
            }
            \context Staff = "LH" {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(staff_group).is_well_formed()
    assert inspect_(staff_group[0][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][2]).get_effective_staff() is staff_group[0]
    assert inspect_(staff_group[0][3]).get_effective_staff() is staff_group[0]
    assert inspect_(staff_group[1][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][2]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][3]).get_effective_staff() is staff_group[1]


def test_agenttools_InspectionAgent_get_effective_staff_03():
    r'''Staff changes work on the last note of a staff.
    '''

    staves = 2 * Staff("c'8 d'8 e'8 f'8")
    staff_group = StaffGroup(staves)
    staff_group.context_name = 'PianoStaff'
    staff_group.is_simultaneous = True
    staff_group[0].name = 'RH'
    staff_group[1].name = 'LH'
    staff_change = indicatortools.StaffChange(staff_group[1])
    attach(staff_change, staff_group[0][-1])

    assert format(staff_group) == stringtools.normalize(
        r'''
        \new PianoStaff <<
            \context Staff = "RH" {
                c'8
                d'8
                e'8
                \change Staff = LH
                f'8
            }
            \context Staff = "LH" {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(staff_group).is_well_formed()


def test_agenttools_InspectionAgent_get_effective_staff_04():
    r'''Redudant staff changes are allowed.
    '''

    staves = 2 * Staff("c'8 d'8 e'8 f'8")
    staff_group = StaffGroup(staves)
    staff_group.context_name = 'PianoStaff'
    staff_group.is_simultaneous = True
    staff_group[0].name = 'RH'
    staff_group[1].name = 'LH'
    staff_change = indicatortools.StaffChange(staff_group[1])
    attach(staff_change, staff_group[0][0])
    staff_change = indicatortools.StaffChange(staff_group[1])
    attach(staff_change, staff_group[0][1])

    assert format(staff_group) == stringtools.normalize(
        r'''
        \new PianoStaff <<
            \context Staff = "RH" {
                \change Staff = LH
                c'8
                \change Staff = LH
                d'8
                e'8
                f'8
            }
            \context Staff = "LH" {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(staff_group).is_well_formed()
    assert inspect_(staff_group[0][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][2]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[0][3]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][0]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][1]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][2]).get_effective_staff() is staff_group[1]
    assert inspect_(staff_group[1][3]).get_effective_staff() is staff_group[1]



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class StemTremolo(AbjadValueObject):
    r'''Stem tremolo.

    ..  container:: example

        **Example 1.** Sixteenth-note tremolo:

        ::

            >>> note = Note("c'4")
            >>> stem_tremolo = indicatortools.StemTremolo(16)
            >>> attach(stem_tremolo, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4 :16

    ..  container:: example

        **Example 2.** Thirty-second-note tremolo:

        ::

            >>> note = Note("c'4")
            >>> stem_tremolo = indicatortools.StemTremolo(32)
            >>> attach(stem_tremolo, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4 :32

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_tremolo_flags',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self, tremolo_flags=16):
        self._default_scope = None
        if isinstance(tremolo_flags, type(self)):
            tremolo_flags = tremolo_flags.tremolo_flags
        tremolo_flags = int(tremolo_flags)
        if not mathtools.is_nonnegative_integer_power_of_two(tremolo_flags):
            message = 'must be nonnegative integer power of 2: {!r}.'
            message = message.format(tremolo_flags)
            raise ValueError(message)
        self._tremolo_flags = tremolo_flags

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats stem tremolo.

        ..  container:: example

            **Example 1.** Sixteenth-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(16)
                >>> print(format(stem_tremolo))
                :16

        ..  container:: example

            **Example 2.** Thirty-second-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(32)
                >>> print(format(stem_tremolo))
                :32

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __str__(self):
        r'''String representation of stem tremolo.

        ..  container:: example

            **Example 1.** Sixteenth-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(16)
                >>> print(str(stem_tremolo))
                :16

        ..  container:: example

            **Example 2.** Thirty-second-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(32)
                >>> print(str(stem_tremolo))
                :32

        Returns string.
        '''
        return ':{!s}'.format(self.tremolo_flags)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.stem_tremolos.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of stem tremolo.

        ..  container:: example

            **Example 1.** Sixteenth-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(16)
                >>> stem_tremolo.default_scope is None
                True

        ..  container:: example

            **Example 2.** Thirty-second-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(32)
                >>> stem_tremolo.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def tremolo_flags(self):
        r'''Gets tremolo flags of stem tremolo.

        ..  container:: example

            **Example 1.** Sixteenth-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(16)
                >>> stem_tremolo.tremolo_flags
                16

        ..  container:: example

            **Example 2.** Thirty-second-note tremolo:

            ::

                >>> stem_tremolo = indicatortools.StemTremolo(32)
                >>> stem_tremolo.tremolo_flags
                32

        Set to nonnegative integer power of 2.

        Defaults to 16.

        Returns nonnegative integer power of 2.
        '''
        return self._tremolo_flags



# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LilyPondVersionToken___format___01():

    lilypond_version_token = lilypondfiletools.LilyPondVersionToken()
    assert isinstance(format(lilypond_version_token), str)
    assert format(lilypond_version_token).count('.') == 2



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_TimespanTimespanTimeRelation___init___01():
    r'''Initializes timespan / timespan time-relation from empty input.
    '''

    time_relation = timespantools.TimespanTimespanTimeRelation()

    assert format(time_relation, 'storage') == stringtools.normalize(
        r'''
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_1.start_offset < timespan_2.start_offset'),
                    ],
                logical_operator='and',
                ),
            )
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.topleveltools import new
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Rotation(AbjadValueObject):
    r'''Rotation operator.

    ..  container:: example:

        ::

            >>> Rotation()
            Rotation(n=0)

    ..  container:: example

        ::

            >>> Rotation(n=1)
            Rotation(n=1)

    Object model of the twelve-tone rotation operator.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_n',
        '_period',
        '_stravinsky',
        )

    ### INITIALIZER ###

    def __init__(self, n=0, period=None, stravinsky=None):
        self._n = int(n)
        if period is not None:
            period = abs(int(period))
            assert 0 < period
        self._period = period
        assert isinstance(stravinsky, (bool, type(None))), repr(stravinsky)
        self._stravinsky = stravinsky

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes rotation and `operator`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [0, 2, 4, 5]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP
    
            Example operators:

            ::

                >>> rotation = Rotation(n=-1)
                >>> transposition = Transposition(n=3)

        ..  container:: example

            Transposition followed by rotation:

            ::

                >>> operator = rotation + transposition
                >>> str(operator)
                'r-1T3'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    f'8
                    g'8
                    af'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because rotation and transposition commute:

            ::

                >>> operator = transposition + rotation
                >>> str(operator)
                'T3r-1'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    f'8
                    g'8
                    af'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }
                
        ..  container:: example

            Returns compound operator:

            ::

                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Rotation(
                            n=-1,
                            ),
                        pitchtools.Transposition(
                            n=3,
                            ),
                        ],
                    )

        '''
        from abjad.tools import pitchtools
        return pitchtools.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        r'''Calls rotation on `argument`.

        ..  container:: example

            Rotates pitch classes:

            ::

                >>> rotation = Rotation(n=1)
                >>> pitch_classes = PitchClassSegment([0, 1, 4, 7])
                >>> rotation(pitch_classes)
                PitchClassSegment([7, 0, 1, 4])

        ..  container:: example

            Rotates pitch classes with Stravinsky-style back-transposition to
            zero:

            ::

                >>> rotation = Rotation(n=1, stravinsky=True)
                >>> pitch_classes = PitchClassSegment([0, 1, 4, 7])
                >>> rotation(pitch_classes)
                PitchClassSegment([0, 5, 6, 9])

        ..  container:: example

            Does not rotate single pitches or pitch-classes:

            ::

                >>> rotation = Rotation(n=1)
                >>> pitch_class = NumberedPitchClass(6)
                >>> rotation(pitch_class)
                NumberedPitchClass(6)

        ..  container:: example

            Periodic rotation:

            ..  todo:: Deprecated.

            ::

                >>> rotation = Rotation(n=1, period=3)
                >>> pitches = PitchSegment("c' d' e' f' g' a' b' c''")
                >>> rotation(pitches)
                PitchSegment("e' c' d' a' f' g' c'' b'")

        ..  container:: example

            Stravinsky-style periodic rotation:

            ..  todo:: Deprecated.

            ::

                >>> rotation = Rotation(
                ...     n=1,
                ...     period=3,
                ...     stravinsky=True,
                ...     )
                >>> pitches = PitchSegment("c' d' e' f' g' a' b' c''")
                >>> rotation(pitches)
                PitchSegment("c' af bf f' df' ef' b' as'")

        Returns new object with type equal to that of `argument`.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, (pitchtools.Pitch, pitchtools.PitchClass)):
            return argument
        if not isinstance(argument, (
            pitchtools.PitchSegment,
            pitchtools.PitchClassSegment,
            )):
            argument = pitchtools.PitchSegment(argument)
        if not self.period:
            return argument.rotate(self.n, stravinsky=self.stravinsky)
        result = new(argument, items=())
        for shard in sequencetools.Sequence(argument).partition_by_counts(
            [self.period],
            cyclic=True,
            overhang=True,
            ):
            shard = type(argument)(shard)
            shard = shard.rotate(self.n, stravinsky=self.stravinsky)
            result = result + shard
        return result

    def __str__(self):
        r'''Gets string representation of operator.

        ..  container:: example

            ::

                >>> str(Rotation())
                'r0'

        ..  container:: example

            ::

                >>> str(Rotation(n=1))
                'r1'

        ..  container:: example

            ::

                >>> str(Rotation(stravinsky=True))
                'rs0'

        ..  container:: example

            ::

                >>> str(Rotation(n=1, stravinsky=True))
                'rs1'

        '''
        string = 'r{}'
        if self.stravinsky:
            string = 'rs{}'
        string = string.format(self.n)
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        operator = markuptools.Markup('r', direction=direction)
        subscript = markuptools.Markup(self.n).sub()
        hspace = markuptools.Markup.hspace(-0.25)
        markup = markuptools.Markup.concat([operator, hspace, subscript])
        return markup

    def _is_identity_operator(self):
        if self.n == 0:
            return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def n(self):
        r'''Gets index of rotation.

        ..  container:: example

            ::

                >>> rotation = Rotation()
                >>> rotation.n
                0

        ..  container:: example

            ::

                >>> rotation = Rotation(n=2)
                >>> rotation.n
                2

        Returns integer.
        '''
        return self._n

    @property
    def period(self):
        r'''Gets period of rotation.

        ..  todo:: Deprecated.

        ..  container:: example

            ::

                >>> rotation = Rotation(n=2, period=3)
                >>> rotation.period
                3

        Returns integer or none.
        '''
        return self._period

    @property
    def stravinsky(self):
        r'''Is true when rotation uses Stravinsky-style back-transposition to
        zero. Otherwise false.

        ..  container:: example

            ::

                >>> rotation = Rotation(n=2, stravinsky=False)
                >>> rotation.stravinsky
                False

        Returns true or false.
        '''
        return self._stravinsky



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import lilypondfiletools
from abjad.tools import markuptools
from abjad.tools import schemetools


def make_mozart_lilypond_file():
    r'''Makes Mozart LilyPond file.
    '''

    score = abjad.demos.mozart.make_mozart_score()
    lilypond_file = lilypondfiletools.LilyPondFile.new(
        music=score,
        global_staff_size=12,
        )
    title = markuptools.Markup(r'\bold \sans "Ein Musikalisches Wuerfelspiel"')
    composer = schemetools.Scheme("W. A. Mozart (maybe?)")
    lilypond_file.header_block.title = title
    lilypond_file.header_block.composer = composer
    lilypond_file.layout_block.ragged_right = True
    lilypond_file.paper_block.markup_system_spacing = schemetools.SchemeAssociativeList(
        ('basic_distance', 8),
        )
    lilypond_file.paper_block.paper_width = 180
    return lilypond_file



# -*- coding: utf-8 -*-
import collections
from abjad.tools.abctools.AbjadObject import AbjadObject


class CyclicTuple(AbjadObject):
    '''A cylic tuple.

    ..  container:: example

        **Example 1.** Initializes from string:

        ::

            >>> tuple_ = datastructuretools.CyclicTuple('abcd')

        ::

            >>> tuple_
            CyclicTuple(['a', 'b', 'c', 'd'])

        ::

            >>> for x in range(8):
            ...     print(x, tuple_[x])
            ...
            0 a
            1 b
            2 c
            3 d
            4 a
            5 b
            6 c
            7 d

    Cyclic tuples overload the item-getting method of built-in tuples.

    Cyclic tuples return a value for any integer index.

    Cyclic tuples otherwise behave exactly like built-in tuples.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_items',
        )

    ### INITIALIZER ###

    def __init__(self, items=None):
        items = items or ()
        items = tuple(items)
        self._items = items

    ### SPECIAL METHODS ###

    def __contains__(self, item):
        r'''Is true when cyclic tuple contains `item`.

        Returns true or false.
        '''
        return self._items.__contains__(item)

    def __eq__(self, argument):
        r'''Is true when `argument` is a tuple with items equal to those of this
        cyclic tuple. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, tuple):
            return self._items == argument
        elif isinstance(argument, type(self)):
            return self._items == argument._items
        return False

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        ..  container:: example

            Gets slice open at right:

            ::

                >>> items = [0, 1, 2, 3, 4, 5]
                >>> tuple_ = datastructuretools.CyclicTuple(items=items)
                >>> tuple_[2:]
                (2, 3, 4, 5)

        ..  container:: example

            Gets slice closed at right:

            ::

                >>> items = [0, 1, 2, 3, 4, 5]
                >>> tuple_ = datastructuretools.CyclicTuple(items=items)
                >>> tuple_[:15]
                (0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2)

        Raises index error when `argument` can not be found in cyclic tuple.

        Returns item.
        '''
        if isinstance(argument, slice):
            if ((argument.stop is not None and argument.stop < 0) or
                (argument.start is not None and argument.start < 0)):
                return self._items.__getitem__(argument)
            else:
                return self._get_slice(argument.start, argument.stop)
        if not self:
            message = 'cyclic tuple is empty: {!r}.'
            message = message.format(self)
            raise IndexError(message)
        argument = argument % len(self)
        return self._items.__getitem__(argument)

    def __hash__(self):
        r'''Hashes cyclic tuple.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(CyclicTuple, self).__hash__()

    def __iter__(self):
        r'''Iterates cyclic tuple.

        Iterates items only once.

        Does not iterate infinitely.
        '''
        return self._items.__iter__()

    def __len__(self):
        r'''Gets length of cyclic tuple.

        Returns nonnegative integer.
        '''
        assert isinstance(self._items, tuple)
        return self._items.__len__()

    def __str__(self):
        r'''Gets string representation of cyclic tuple.

        ..  container:: example

            Gets string:

            ::

                >>> str(datastructuretools.CyclicTuple('abcd'))
                '(a, b, c, d)'

        ..  container:: example

            Gets string:

            ::

                >>> str(datastructuretools.CyclicTuple([1, 2, 3, 4]))
                '(1, 2, 3, 4)'

        Returns string.
        '''
        if self:
            contents = [str(item) for item in self._items]
            contents = ', '.join(contents)
            string = '({!s})'.format(contents)
            return string
        return '()'

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[list(self._items)],
            )

    def _get_slice(self, start_index, stop_index):
        if stop_index is not None and 1000000 < stop_index:
            stop_index = len(self)
        result = []
        if start_index is None:
            start_index = 0
        if stop_index is None:
            indices = range(start_index, len(self))
        else:
            indices = range(start_index, stop_index)
        result = [self[n] for n in indices]
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def items(self):
        r'''Gets items in cyclic tuple.

        ..  container:: example

            Gets items:

            ::

                >>> tuple_ = datastructuretools.CyclicTuple('abcd')
                >>> tuple_.items
                ('a', 'b', 'c', 'd')

        ..  container:: example

            Gets items:

            ::

                >>> tuple_ = datastructuretools.CyclicTuple([1, 2, 3, 4])
                >>> tuple_.items
                (1, 2, 3, 4)

        Returns tuple.
        '''
        return self._items


collections.Sequence.register(CyclicTuple)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_datastructuretools_OrdinalConstant___builtins___01():

    assert Left < Right
    assert Down < Center < Up


def test_datastructuretools_OrdinalConstant___builtins___02():

    assert pytest.raises(Exception, 'Down < Right')



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_selectiontools_Selection__withdraw_from_crossing_spanners_01():
    r'''Withdraw components from crossing spanners.
    No spanners cross voice.
    '''

    voice = Voice(r'''
        {
            c'8 [ \startTrillSpan
            d'8 ]
        }
        {
            e'8 (
            f'8 ) \stopTrillSpan
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8 ]
            }
            {
                e'8 (
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    voice_selection = selectiontools.Selection([voice])
    voice_selection._withdraw_from_crossing_spanners()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8 ]
            }
            {
                e'8 (
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_selectiontools_Selection__withdraw_from_crossing_spanners_02():
    r'''Withdraw logical-voice-contiguous components from crossing spanners.
    '''

    voice = Voice(r'''
        {
            c'8 [ \startTrillSpan
            d'8 ]
        }
        {
            e'8 (
            f'8 ) \stopTrillSpan
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8 ]
            }
            {
                e'8 (
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    voice[:1]._withdraw_from_crossing_spanners()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                d'8 ]
            }
            {
                e'8 ( \startTrillSpan
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_selectiontools_Selection__withdraw_from_crossing_spanners_03():
    r'''Withdraw logical-voice-contiguous components from crossing spanners.
    Operation leaves score tree in weird state.
    Both slur and trill become discontiguous.
    '''

    voice = Voice(r'''
        {
            c'8 [ \startTrillSpan
            d'8 ]
        }
        {
            e'8 (
            f'8 ) \stopTrillSpan
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8 ]
            }
            {
                e'8 (
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(voice)
    leaves[2:3]._withdraw_from_crossing_spanners()
    assert not inspect_(voice).is_well_formed()


# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Beam_direction_01():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    beam = Beam(direction=Up)
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ^ [
            d'8
            e'8
            f'8 ]
            g'2
        }
        '''
        )


def test_spannertools_Beam_direction_02():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    beam = Beam(direction=Down)
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 _ [
            d'8
            e'8
            f'8 ]
            g'2
        }
        '''
        )


def test_spannertools_Beam_direction_03():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    beam = Beam(direction=Center)
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 - [
            d'8
            e'8
            f'8 ]
            g'2
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension__initialize_by_reference_01():

    chord_suspension = tonalanalysistools.ChordSuspension(4, 3)
    u = tonalanalysistools.ChordSuspension(chord_suspension)

    assert chord_suspension is not u
    assert chord_suspension == u



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_is_misfilled_01():

    measure = Measure((3, 4), "c' d' e'")
    assert not measure.is_misfilled


def test_scoretools_Measure_is_misfilled_02():

    measure = Measure((3, 4), "c' d' e' f'")
    assert measure.is_misfilled



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTTOCItem(TreeNode):
    r'''A ReST TOC item.

    ::

        >>> item = documentationtools.ReSTTOCItem(text='api/index')
        >>> item
        ReSTTOCItem(
            text='api/index'
            )

    ::

        >>> print(item.rest_format)
        api/index

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, name=None, text='foo'):
        TreeNode.__init__(self, name)
        self.text = text

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        return [self.text]

    ### PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        r'''ReST format of ReST TOC item.

        Returns string.
        '''
        return '\n'.join(self._rest_format_contributions)

    @property
    def text(self):
        r'''Gets and sets text of ReST TOC item.

        Returns string.
        '''
        return self._text

    @text.setter
    def text(self, arg):
        assert stringtools.is_string(arg)
        arg = arg.strip()
        self._text = arg



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_WellformednessManager_check_overlapping_glissandi_01():

    staff = Staff("c'4 d'4 e'4 f'4")
    attach(Glissando(), staff[:2])
    attach(Glissando(), staff[1:3])

    assert inspect_(staff).is_well_formed()


def test_systemtools_WellformednessManager_check_overlapping_glissandi_02():

    staff = Staff("c'4 d'4 e'4 f'4")
    attach(Glissando(), staff[:2])
    attach(Glissando(), staff[:3])

    assert not inspect_(staff).is_well_formed()
